package com.trs.ggzyexchange.schedul;


import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.trs.ggzyexchange.api.vo.Result;
import com.trs.ggzyexchange.controller.BaseController;
import com.trs.ggzyexchange.entity.GcPerformance;
import com.trs.ggzyexchange.entity.GcPubContract;
import com.trs.ggzyexchange.entity.ScheduledTash;
import com.trs.ggzyexchange.entity.TenderDetailsVo;
import com.trs.ggzyexchange.enums.ClassIfy;
import com.trs.ggzyexchange.enums.JobLock;
import com.trs.ggzyexchange.service.GcNoticeandfileService;
import com.trs.ggzyexchange.service.GcPubContractService;
import com.trs.ggzyexchange.service.ScheduledTashService;
import com.trs.ggzyexchange.service.TenderDetailsVoService;
import com.trs.ggzyexchange.utils.TenderDetailsVoUtils;
import org.apache.commons.lang3.ObjectUtils;
import org.quartz.Job;
import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * <p>
 * 合同签订 前端控制器
 * </p>
 *
 * @author trs
 * @since 2023-02-23
 */
@Component
public class GcPubContractSchedul implements Job {

    private Logger logger = LoggerFactory.getLogger(GcPubContractSchedul.class);

    @Autowired
    private GcPubContractService gcPubContractService;

    @Autowired
    private TenderDetailsVoService tenderDetailsVoService;

    @Autowired
    private GcNoticeandfileService gcNoticeandfileService;

    @Autowired
    private ScheduledTashService scheduledTashService;

    public void queryPageListUpdateEs() throws JsonProcessingException {
        //查询当前在定时任务表当中的时间
        QueryWrapper<ScheduledTash> scheduledTashQueryWrapper = new QueryWrapper<ScheduledTash>();
        scheduledTashQueryWrapper.eq("table_name_value","gc_pub_contract");
        Page<ScheduledTash> pageScheduledTash = new Page<ScheduledTash>(1, 1);
        Page<ScheduledTash> page = scheduledTashService.page(pageScheduledTash, scheduledTashQueryWrapper);
        List<ScheduledTash> records = page.getRecords();
        ScheduledTash scheduledTash = null;
        if(records.size() > 0){
            scheduledTash = records.get(0);
        }
        Long time = null;
        if(ObjectUtils.isEmpty(scheduledTash)){
            try {
                //查询表gcNoticeandfile表当中的最新时间
                time = gcPubContractService.maxDataTimeStamp();
                ScheduledTash scheduledTashTash = new ScheduledTash();
                scheduledTashTash.setTime(time);
                scheduledTashTash.setTableNameValue("gc_pub_contract");
                scheduledTashTash.setUpdateTime(new Date());
                scheduledTashTash.setCreateTime(new Date());
                scheduledTashService.saveOrUpdate(scheduledTashTash,scheduledTashQueryWrapper);
            }catch (Exception e){
                e.printStackTrace();
            }
        }else {
            time = scheduledTash.getTime();
        }
        try {
            //查询表gcNoticeandfile表当中的最新时间
            Long newTime = gcPubContractService.maxDataTimeStamp();
            ScheduledTash scheduledTashTash = new ScheduledTash();
            scheduledTashTash.setTime(newTime);
            scheduledTashTash.setTableNameValue("gc_pub_contract");
            scheduledTashTash.setUpdateTime(new Date());
            scheduledTashService.saveOrUpdate(scheduledTashTash,scheduledTashQueryWrapper);
        }catch (Exception e){
            e.printStackTrace();
        }
        if(!ObjectUtils.isEmpty(time)){
            QueryWrapper<GcPubContract> gcPerformanceQueryWrapper = new QueryWrapper<GcPubContract>();
            gcPerformanceQueryWrapper.gt("DATA_TIMESTAMP",time);
            int count = gcPubContractService.count(gcPerformanceQueryWrapper);
            Integer pageSize = 1000;
            Integer pageNum = (int)Math.ceil((double)count/pageSize);
            for(int i = 1; i <= pageNum;i++){
                List<TenderDetailsVo> tenderDetailsVoListAll = getTenderDetailsVoListAll(i, pageSize,time);
                gcNoticeandfileService.parseTenderDetailsVo(tenderDetailsVoListAll);
            }
        }
    }

    public List<TenderDetailsVo> getTenderDetailsVoListAll(Integer pageNo,Integer pageSize,Long time){
        Page<GcPubContract> page = new Page<GcPubContract>(pageNo, pageSize);
        IPage<TenderDetailsVo> tenderDetailsVoIPage = gcPubContractService.queryGcBidcandidateAllListByTime(page,time);
        for (TenderDetailsVo record : tenderDetailsVoIPage.getRecords()) {
           try {
               record.setNoticeTypeCode(ClassIfy.CONTRACT_PUBLICITY.getCode());
               record.setNoticeTypeName(ClassIfy.CONTRACT_PUBLICITY.getName());
               record.setTransactionTypeCode(ClassIfy.ENGINEERING_CONSTRUCTION.getCode());
               record.setTransactionTypeName(ClassIfy.ENGINEERING_CONSTRUCTION.getName());
               record = tenderDetailsVoService.getTenderDetailsVoAllOrther(record);
               TenderDetailsVoUtils.isNull(record);
           }catch (Exception e){
               e.printStackTrace();
           }
        }
        return tenderDetailsVoIPage.getRecords();
    }

    @Override
    public void execute(JobExecutionContext jobExecutionContext) throws JobExecutionException {
        try {
            if(JobLock.GcPubContractFlag){
                JobLock.GcPubContractFlag = false;
                logger.info("执行GcPubContract", new SimpleDateFormat().format(new Date()));
                try {
                    queryPageListUpdateEs();
                } catch (Exception e) {
                    e.printStackTrace();
                }
                JobLock.GcPubContractFlag = true;
            }else{
                logger.info("不执行GcPubContract", new SimpleDateFormat().format(new Date()));
            }
        }catch (Exception e){
            e.printStackTrace();
        }
    }
}
